
//
// MailDirCrawler.cs
//
// Copyright (C) 2005 Noam Rathaus, Aviram Jenik
//
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

using System;
using System.Collections;
using System.IO;

using Beagle.Util;
using Beagle.Daemon;

namespace Beagle.Daemon.MailDirDriver {
	
	class MailDirCrawler {
		ArrayList roots = new ArrayList ();

		Hashtable last_write_time_cache = new Hashtable ();
		ArrayList emails = new ArrayList ();
		
		public MailDirCrawler (params string[] paths)
		{
			foreach (string p in paths) {
				roots.Add (p);
			}
		}

		private bool FileIsInteresting (FileInfo file)
		{
			DateTime cached_time = new DateTime ();
			if (last_write_time_cache.Contains (file.FullName))
				cached_time = (DateTime) last_write_time_cache [file.FullName];
			
			last_write_time_cache [file.FullName] = file.LastWriteTime;
			
			return cached_time < file.LastWriteTime;
		}

		public void Crawl ()
		{
			emails.Clear ();

			Queue pending = new Queue ();

			foreach (string root in roots)
				pending.Enqueue (new DirectoryInfo (root));

			Logger.Log.Info ("Crawls pending: {0}", pending.Count);

			while (pending.Count > 0) {

				DirectoryInfo dir = pending.Dequeue () as DirectoryInfo;

				Logger.Log.Info ("[MailDir] Looking into directory: {0}", dir);

				foreach (DirectoryInfo subdir in dir.GetDirectories ())
				{
					string noindex = Path.Combine (subdir.FullName, ".noindex");
				 
				 	if (!File.Exists(noindex))
					{
						pending.Enqueue (subdir);
					}
				}

				foreach (FileInfo file in dir.GetFiles ()) {
					// FIXME: Is this the only extension list?
					if (file.Extension == ".index" || file.Extension == ".ids" ||
					    file.Extension == ".sorted" || file.Extension == ".temp"
							)
					{
						Logger.Log.Info ("Ignoring {0}", file);
					}
					else
					{
						FileIsInteresting (file);
						emails.Add(file);
					}
				}
			}
		}

		public ICollection Emails {
			get { return emails; } 
		}

	}
}
